@@ -5,7 +5,8 @@
 #' accuracy, user's accuracy, and area. Includes precision estimates.
 #' 
 #' Argument \code{Nh_strata} must be named to explicitly and clearly identify the stratum that 
-#' each entry refers to.
+#' each entry refers to. The names of \code{Nh_strata} are expected to match the strata class 
+#' labels of argument \code{s}.
 #' 
 #' In the error matrix returned, the entries corresponding to no observed cases will present
 #' \code{NA} rather than 0. This is to emphasize the difference between the absence of cases
@@ -15,7 +16,8 @@
 #' @param s character vector. Strata class labels. The object will be coerced to factor.
 #' @inheritParams olofsson
 #' @param Nh_strata numeric vector. Number of pixels forming each stratum. It must be named (see details).
-#' @param order character vector. Order of the classes to be displayed in the results.
+#' @param order (optional) character vector. Classes to be displayed in the results and their sequence.
+#' If missing, equal to \code{sort(unique(c(r,m)))}.
 #'
 #' @return A list with the estimates and error matrix.
 #' \item{OA}{overall accuracy}
@@ -37,10 +39,12 @@
 #' # Numerical example in Stheman (2014)
 #' s<-c(rep("A",10), rep("B",10), rep("C",10), rep("D",10))
 #' m<-c(rep("A",7), rep("B",3), "A", rep("B",11), rep("C",6), "B", "B", rep("D",10))
-#' r<-c(rep("A",5), "C", "B", "A", "B", "C", "A", rep("B",5),
-#'               "A", "A", "B", "B", rep("C",5), "D", "D", "B", "B", "A", rep("D",7), "C", "C", "B")
-#' Nh_strata<-c("A"=40000, "B"=30000, "C"=20000, "D"=10000) 
+#' r<-c(rep("A",5), "C", "B", "A", "B", "C", "A", rep("B",5), "A", "A", "B", 
+#'      "B", rep("C",5), "D", "D", "B", "B", "A", rep("D",7), "C", "C", "B")
+#' Nh_strata<-c(40000, 30000, 20000, 10000)
+#' names(Nh_strata)<-c("A", "B", "C", "D")
 #' e<-stehman2014(s, r, m, Nh_strata)
+#' 
 #' e$area[1]                  # Proportion of area of class A (compare with paper in p. 4932)
 #' e$area[3]                  # Proportion of area of class C (p. 4932)
 #' e$OA                       # Overall accuracy (p. 4932)
@@ -53,22 +57,28 @@
 #' e$SEua[2]                  # SE for user's accuracy of class B (p. 4936)
 #' e$SEpa[2]                  # SE for producer's accuracy of class B (p. 4936)
 #' 
-#'
 #' # change class order
-#' stehman2014(s, m, r, Nh_strata, order=c("D","C","B","A"))
+#' stehman2014(s, r, m, Nh_strata, order=c("D","C","B","A"))
 #' 
-#' 
-#' # When the number of strata differs from the number of map classes
-#' s<-c(rep("A",5), rep("AA",5), rep("B",10), rep("C",10), rep("D",10))
-#' m<-c(rep("A",4), "B", rep("A",4), "B", "A", rep("B",11), rep("C",6), "B", "B", rep("D",10))
-#' r<-c(rep("A",4), "C", rep("A",4), "C", "A", rep("B",5),
-#'      "A", "A", "B", "B", rep("C",5), "D", "D", "B", "B", "A", rep("D",7), "C", "C", "B")
-#' Nh_strata<-c("A"=20000, "AA"=20000, "B"=30000, "C"=20000, "D"=10000)
+#' # the number (and name) of strata and map classes may differ
+#' s<-c(rep("a",5), rep("aa",5), rep("b",10), rep("c",10), rep("d",10))
+#' Nh_strata<-c("a"=20000, "aa"=20000, "b"=30000, "c"=20000, "d"=10000)
 #' stehman2014(s, r, m, Nh_strata)
 #' 
+#' # m (map) may include classes not found in r (reference)
+#' m<-c(rep("A",7), rep("B",3), "A", rep("B",11), rep("C",6), "B", "B", rep("D",9), "XX")
+#' stehman2014(s, r, m, Nh_strata)
 #' 
+#' # r (reference) may include classes not found in m (map)
+#' m<-c(rep("A",7), rep("B",3), "A", rep("B",11), rep("C",6), "B", "B", rep("D",10))
+#' r<-c(rep("A",5), "C", "B", "A", "B", "C", "A", rep("B",5), "A", "A", "B",
+#'      "B", rep("C",5), "D", "D", "B", "B", "A", rep("D",7), "C", "C", "YY")
+#' stehman2014(s, r, m, Nh_strata)
+#' 
+#' # can add classes not found neither in r nor m
+#' stehman2014(s, r, m, Nh_strata, order=c("A","B","C","D","YY","ZZ"))
 #' @export
-stehman2014<-function(s, r, m, Nh_strata, margins=TRUE, order = sort(unique(r))){
+stehman2014<-function(s, r, m, Nh_strata, margins=TRUE, order){
 
   # check arguments
   s<-unlist(s)
@@ -81,10 +91,18 @@ stehman2014<-function(s, r, m, Nh_strata, margins=TRUE, order = sort(unique(r)))
   # compare arguments
   .check_labels(names(Nh_strata), s)
   Nh_strata<-Nh_strata[names(Nh_strata)%in%s] # keep only the strata found in s
-  .check_labels(r, m)
   .check_length(s, r, m)
 
   # covert arguments
+  if(missing(order)){
+    order<-sort(union(r, m))
+  }else{
+    temp<-sort(setdiff(union(r, m), order))
+    if(length(temp)>0){
+      message<-"Argument 'order' must include the following label(s): "
+      stop(message, paste(temp, collapse = "; "), call. = FALSE)
+    }
+  }
   s<-factor(s, levels = names(Nh_strata))
   r<-factor(r, levels = order)
   m<-factor(m, levels = order)
@@ -229,6 +247,7 @@ stehman2014<-function(s, r, m, Nh_strata, margins=TRUE, order = sort(unique(r)))
   }
   U<-R_U_numerator/R_U_dominator
   names(U)<-classes2
+  U[is.nan(U)]<-NA # when r has a class not found in m
   
   # producer's accuracy
   R_P_numerator<-NULL
@@ -241,6 +260,7 @@ stehman2014<-function(s, r, m, Nh_strata, margins=TRUE, order = sort(unique(r)))
   }
   P<-R_P_numerator/R_P_dominator
   names(P)<-classes2
+  P[is.nan(P)]<-NA # when m has a class not found in r
 
   
   # Standard error (SE) for user's accuracy
